package source 
{
	import flash.display.BitmapData;
	import flash.display.DisplayObject;
	import flash.geom.ColorTransform;
	import flash.geom.Matrix;
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import flash.utils.ByteArray;
	import net.flashpunk.graphics.Backdrop;
	import net.flashpunk.graphics.Text;
	import net.flashpunk.Sfx;
	
	import net.flashpunk.Entity;
	import net.flashpunk.FP;
	import net.flashpunk.World;
	import net.flashpunk.graphics.Tilemap;
	
	import net.flashpunk.utils.Input;
	import net.flashpunk.utils.Draw;
	
	import flash.display.BlendMode;
	
	import source.Objects.*;
	import source.Solids.*;
	/**
	 * ...
	 * @author Noel Berry
	 */
	public class Game extends World
	{
		[Embed(source = "../assets/graphics/backwall.png")] private static var imgBackWall:Class;
		[Embed(source = "../assets/graphics/outroback.png")] private static var imgOutroBack:Class;
		[Embed(source = "../assets/graphics/outrowall.png")] private static var imgOutroWall:Class;
		
		[Embed(source = "../assets/graphics/screencover.png")] private static var imgScreenCover:Class;
		private var sprScreenCover:DisplayObject = new imgScreenCover();
		
		[Embed(source = "../assets/sounds/Cave.mp3")] private static var sndCave:Class;
		private var sndOCave:Sfx = new Sfx(sndCave);
		[Embed(source = "../assets/sounds/CaveEnding.mp3")] private static var sndCaveEnding:Class;
		private var sndOCaveEnding:Sfx = new Sfx(sndCaveEnding, endOutro);
		
		[Embed(source = '../assets/levels/Level1.oel',  mimeType = "application/octet-stream")] public static var level1: Class;
		[Embed(source = '../assets/levels/Level2.oel',  mimeType = "application/octet-stream")] public static var level2: Class;
		[Embed(source = '../assets/levels/Level3.oel',  mimeType = "application/octet-stream")] public static var level3: Class;
		[Embed(source = '../assets/levels/Level4.oel',  mimeType = "application/octet-stream")] public static var level4: Class;
		[Embed(source = '../assets/levels/Level5.oel',  mimeType = "application/octet-stream")] public static var level5: Class;
		[Embed(source = '../assets/levels/Level6.oel',  mimeType = "application/octet-stream")] public static var level6: Class;
		[Embed(source = '../assets/levels/Level7.oel',  mimeType = "application/octet-stream")] public static var level7: Class;
		[Embed(source = '../assets/levels/LevelLeprechaun.oel',  mimeType = "application/octet-stream")] public static var levelLep: Class;
		[Embed(source = '../assets/levels/Outro.oel',  mimeType = "application/octet-stream")] public static var outro: Class;
		[Embed(source = '../assets/levels/Outro2.oel',  mimeType = "application/octet-stream")] public static var outro2: Class;
		[Embed(source = '../assets/graphics/tileset.png')] public var imgTileset:Class;
		
		public static var levels:Array = new Array(level1, level2, level3, level4, level6, level5, level7, levelLep, outro, outro2);
		public var tileset:Tilemap;
		public var tilesetBelow:Tilemap;
		public var tileAboveEntity:Entity;
		public var tileBelowEntity:Entity;
		
		//timer so that reset level doesn't happen right away...
		public var reset:int = 120;
		
		public static var lights:BitmapData = new BitmapData(FP.width, FP.height, true, 0x00000000);
		
		public static var cycleSolids:Boolean = false;
		public static var tilesetPos:Point = new Point();
		
		private var playedEndingMusic:Boolean = false;
		
		public static var cover:Number = 0;
		public static const coverRate:Number = 0.025;
		
		public static var gotoLepRoom:Boolean = false;
		
		private var outroText:Text;
		private var outroTextEntity:Entity;
		
		private var outroDeathText:Text;
		private var outroDeathTextEntity:Entity;
		
		override public function begin():void {
			Global.level = 0;
			nextlevel();
			sndOCave.loop();
		}
		
		override public function end():void
		{
			sndOCave.stop();
			sndOCaveEnding.stop();
		}
		
		override public function update():void {
			if (!Global.paused) { super.update(); }
			
			if (cover > 0 && !Global.finished)
			{
				if (Global.level - 1 == 0)
				{
					cover -= coverRate / 4;
				}
				else
				{
					cover -= coverRate;
				}
			}
			
			if (levels[Global.level - 1] == outro2)
			{
				if (outroTextEntity)
				{
					outroTextEntity.x = FP.camera.x + FP.width / 4 - outroText.width / 2;
					outroTextEntity.y = Math.max(outroTextEntity.y - 1, FP.camera.y + 62 - outroText.height / 2);
				}
				if (outroDeathTextEntity)
				{
					outroDeathTextEntity.x = FP.camera.x + FP.width / 4 - outroText.width / 2;
					outroDeathTextEntity.y = Math.max(outroDeathTextEntity.y - 1, FP.camera.y + 62 - outroDeathText.height / 2) + 20;
				}
				if (sndOCave.volume > 0)
				{
					sndOCave.volume -= 0.01;
				}
				else if(!sndOCaveEnding.playing && !playedEndingMusic)
				{
					sndOCaveEnding.play();
					playedEndingMusic = true;
				}
			}
			
			//if we should restart
			if (Global.restart) { 
				
				//make a timer so it isn't instant
				reset--;
				if (reset == 0 || Input.released(Global.keyRestart)) {
					//restart level
					restartlevel();
					//set restart to false
					Global.restart = false;
					//reset our timer
					reset = 60;
				}
			}
			
			if (Global.finished) {
				cover += coverRate;
				if (cover >= 1)
				{
					nextlevel();
				}
			}
		}
		
		override public function render():void
		{
			super.render();
			if (cycleSolids)
			{
				tileAboveEntity.visible = false;
				tileBelowEntity.visible = false;
				tilesetPos.x += Global.player.mMaxspeed.x;
				tilesetPos.x = tilesetPos.x % tileset.width;
				tileset.render(new Point(tilesetPos.x, tilesetPos.y), FP.camera);
				tileset.render(new Point(tilesetPos.x - tileset.width, tilesetPos.y), FP.camera);
			}
			
			lights.fillRect(lights.rect, 0xFF000000);
			var lightVector:Vector.<Entity> = new Vector.<Entity>();
			getClass(Light, lightVector);
			getClass(DownLight, lightVector);
			for each(var l:Light in lightVector)
			{
				l.render();
			}
			lightVector = new Vector.<Entity>()
			getClass(Floater, lightVector);
			for each(var f:Floater in lightVector)
			{
				f.lightDraw();
			}
			lightVector = new Vector.<Entity>()
			getClass(Flyer, lightVector);
			for each(var f1:Flyer in lightVector)
			{
				f1.lightDraw();
			}
			lightVector = new Vector.<Entity>()
			getClass(Sitter, lightVector);
			for each(var s:Sitter in lightVector)
			{
				s.lightDraw();
			}
			lights.draw(sprScreenCover, null, new ColorTransform(1, 1, 1, 0.5), BlendMode.DARKEN);
			
			var a:Number = 1;
			if (levels[Global.level - 1] == outro)
			{
				a = (Global.player.x - FP.width / 2) / (FP.width / 2);
			}
			if (levels[Global.level - 1] == outro2)
			{
				a = 0;
			}
			if (levels[Global.level - 1] == levelLep)
			{
				lights.draw(sprScreenCover, null, new ColorTransform(1, 1, 1, 1), BlendMode.ADD);
			}
			FP.buffer.draw(lights, null, new ColorTransform(1, 1, 1, a), BlendMode.MULTIPLY);
			
			Draw.rect(FP.camera.x, FP.camera.y, FP.width, FP.height, 0x000000, cover);
			
			if (levels[Global.level - 1] == outro2)
			{
				outroText.render(new Point(outroTextEntity.x, outroTextEntity.y), FP.camera);
				//outroDeathText.render(new Point(outroDeathTextEntity.x, outroDeathTextEntity.y), FP.camera);
			}
		}
		
		public function loadlevel():void 
		{	
			var file:ByteArray = new levels[Global.level-1];
			var str:String = file.readUTFBytes( file.length );
			var xml:XML = new XML(str);
			
			var e:Entity;
			var o:XML;
			var n:XML;
			
			FP.width = xml.width;
			FP.height = xml.height;		
			
			if (levels[Global.level - 1] != outro && levels[Global.level-1] != outro2)
			{
				addGraphic(new Backdrop(imgBackWall), 1000);
			}
			else
			{
				addGraphic(new Backdrop(imgOutroBack), 1000);
				if (levels[Global.level - 1] == outro)
				{
					addGraphic(new Backdrop(imgOutroWall), 1000);
				}
				else
				{
					outroTextEntity = addGraphic(outroText = new Text("You escaped!"), -200, FP.camera.x + FP.width / 2, FP.camera.y + FP.height);
					outroText.color = 0x000000;
					outroDeathTextEntity = addGraphic(outroDeathText = new Text("Deaths: " + Global.deaths), -200, FP.camera.x + FP.width / 2, FP.camera.y + FP.height);
					outroDeathText.color = 0x000000;
				}
				add(new Sun());
			}
			//add the tileset to the world
			add(tileBelowEntity = new Entity(0, 0, tilesetBelow = new Tilemap(imgTileset, FP.width, FP.height, Global.grid, Global.grid)));
			tileBelowEntity.layer = 100;
			add(tileAboveEntity = new Entity(0, 0, tileset = new Tilemap(imgTileset, FP.width, FP.height, Global.grid, Global.grid)));
			tileAboveEntity.layer = -100;
			
			//add the view, and the player
			add(Global.player = new Player(xml.objects[0].player.@x, xml.objects[0].player.@y));
			
			//set the view to follow the player, within no restraints, and let it "stray" from the player a bit.
			//for example, if the last parameter was 1, the view would be static with the player. If it was 10, then
			//it would trail behind the player a bit. Higher the number, slower it follows.
			add(Global.view = new View(Global.player as Entity, new Rectangle(0,0,FP.width,FP.height), 10));
			
			//add tiles
			const columns:int = 6;
			if (str.search("</tilesBelow>") > 0) 
			{
				for each (o in xml.tilesBelow[0].tile)
				{
					//place the tiles in the correct position
					//NOTE that you should replace the "5" with the amount of columns in your tileset!
					tilesetBelow.setTile(o.@x / Global.grid, o.@y / Global.grid, (columns * (o.@ty/Global.grid)) + (o.@tx/Global.grid));
				}
			}
			
			if (str.search("</tilesAbove>") > 0) 
			{
				for each (o in xml.tilesAbove[0].tile) 
				{
					//place the tiles in the correct position
					//NOTE that you should replace the "5" with the amount of columns in your tileset!
					tileset.setTile(o.@x / Global.grid, o.@y / Global.grid, (columns * (o.@ty/Global.grid)) + (o.@tx/Global.grid));
				}
			}
			
			if (str.search("</floors>") > 0) 
			{
				//place the solids
				for each (o in xml.floors[0].rect) 
				{
					//place flat solids, setting their position & width/height
					add(new Solid(o.@x, o.@y, o.@w, o.@h, 1));
				}
			}
			if (str.search("</greenfloors>") > 0) 
			{
				//place the solids
				for each (o in xml.greenfloors[0].rect) 
				{
					//place flat solids, setting their position & width/height
					add(new Solid(o.@x, o.@y, o.@w, o.@h, 0));
				}
			}
			
			if(str.search("<slopes>") > 0) {
				for each (o in xml.slopes[0].slope) {
					//place a slope
					add(new Slope(o.@x, o.@y, o.@type));
				}
			}
			//place a crate
			for each (o in xml.objects[0].crate) { add(new Crate(o.@x, o.@y)); }
			
			//place a moving platform
			for each (o in xml.objects[0].moving) { add(new Moving(o.@x, o.@y)); }
			
			//place a moving platform
			for each (o in xml.objects[0].spikes) { add(new Spikes(o.@x, o.@y)); }
			
			//place electricity
			for each (o in xml.objects[0].electricity) {
				var p:Point;
				//get the end point of the electricity (via nodes)
				for each (n in o.node)
				{
					p = new Point(n.@x, n.@y);
				}
				
				//add electricity to the world
				add(new Electricity(o.@x, o.@y, p.x, p.y));
			}
			
			//add the door!
			for each (o in xml.objects[0].door) { add(new Door(o.@x, o.@y)); }
			
			for each (o in xml.objects[0].floater) { add(new Floater(o.@x, o.@y)); }
			
			for each (o in xml.objects[0].worm) { add(new Worm(o.@x, o.@y)); }
			
			for each (o in xml.objects[0].faller) { add(new Faller(o.@x)); }
			
			for each (o in xml.objects[0].flyer) { add(new Flyer(o.@x, o.@y)); }
			
			for each (o in xml.objects[0].downlight) { add(new DownLight(o.@x, o.@y)); }
			
			for each (o in xml.objects[0].sitter) { add(new Sitter(o.@x, o.@y)); }
			
			for each (o in xml.objects[0].leprechaun) { add(new Leprechaun(o.@x, o.@y)); }
		}
		
		
		/**
		 * Loads up the next level (removes all entities of the current world, increases Global.level, calls loadlevel)
		 * @return	void
		 */
		public function nextlevel():void
		{
			removeAll();
			
			Global.finished = false;
			
			cycleSolids = false;
			
			cover = 1;
			
			if (levels[Global.level - 1] == outro2)
			{
				FP.world = new Menu();
				return;
			}
			
			if (Global.level < levels.length) { Global.level ++; }
			
			if (gotoLepRoom)
			{
				Global.level = levels.length - 2;
				gotoLepRoom = false;
			}
			else if (Global.level == levels.length - 2)
			{
				Global.level++;
			}
			
			loadlevel();
			
			/*var v:Vector.<Entity> = new Vector.<Entity>();
			getClass(Solid, v); FP.console.log(v.length);
			for each(var s:Entity in v)
			{
				if (s is Solid)
				{
					(s as Solid).setBoundaries((s as Solid).width, (s as Solid).height);
				}
			}*/
		}
		
		
		/**
		 * Reloads the current level
		 * @return	void
		 */
		public function restartlevel():void
		{
			if (levels[Global.level - 1] == outro2 || levels[Global.level - 1] == outro)
			{
				return;
			}
			removeAll();
			loadlevel();

			
			//increase deaths
			Global.deaths ++;
		}
		
		public function endOutro():void
		{
			Global.finished = true;
		}
		
	}

}