package source.Objects 
{
	import flash.geom.Matrix;
	import flash.geom.Point;
	import net.flashpunk.graphics.Spritemap;
	import net.flashpunk.FP;
	import net.flashpunk.masks.Pixelmask;
	import net.flashpunk.Sfx;
	import source.Game;
	import flash.display.BlendMode;
	import net.flashpunk.utils.Draw;
	import source.View;
	/**
	 * ...
	 * @author Time
	 */
	public class Floater extends Enemy
	{
		[Embed(source = "../../assets/sounds/FloaterFloat.mp3")] private var sndFloaterFloat:Class;
		private var sndOFloaterFloat:Sfx = new Sfx(sndFloaterFloat);
		[Embed(source = "../../assets/sounds/Floater.mp3")] private var sndFloater:Class;
		private var sndOFloater:Sfx = new Sfx(sndFloater);
		
		[Embed(source = "../../assets/graphics/floaterMask.png")] private var imgFloaterMask:Class;
		[Embed(source = "../../assets/graphics/floaterLights.png")] private var imgFloaterLights:Class;
		private var sprFloaterLights:Spritemap = new Spritemap(imgFloaterLights, 64, 60);
		[Embed(source = "../../assets/graphics/floater.png")] private var imgFloater:Class;
		private var sprFloater:Spritemap = new Spritemap(imgFloater, 64, 60, endAnim);
		
		private var range:int = 250;
		
		private const waitTimeMax:int = 90;
		private var waitTime:int = 0;
		private var falling:Boolean = false;
		
		private const accelMax:Point = new Point(0.6, 0.05);
		private var prevSpeed:Point = new Point();
		
		public function Floater(_x:int, _y:int) 
		{
			super(_x + 32, _y + 30);
			graphic = sprFloater;
			sprFloater.add("Float", [1, 2], 0.1);
			sprFloater.add("Fall", [0]);
			sprFloater.add("Hurt", [3, 0, 3, 0, 3], 0.3);
			sprFloater.play("Float");
			sprFloater.centerOO();
			
			sprFloaterLights.centerOO();
			
			mMaxspeed.x = 0.75;
			
			type = "Floater";
			
			mask = new Pixelmask(imgFloaterMask, -32, -30);
			
			sndOFloaterFloat.loop();
		}
		
		override public function removed():void
		{
			sndOFloaterFloat.stop();
		}
		
		override public function update():void
		{
			super.update();
			
			var player:Player = FP.world.nearestToEntity("Player", this) as Player;
			
			acceleration = new Point();
			sndOFloaterFloat.volume = 0;
			if (falling)
			{
				sprFloater.play("Fall");
				gravity();
				if (collide("Solid", x, y + 1))
				{
					waitTime = waitTimeMax;
					sndOFloater.play(Math.max(1 - FP.distance(x, y, player.x, player.y) / 1000, 0) * Math.min(Math.abs(speed.y / 2), 1));
					falling = false;
					View.time = 5 * prevSpeed.y;
				}
				else
				{
					acceleration.x = FP.sign(player.x - x) * Math.abs(accelMax.x);
				}
			}
			else if (waitTime > 0)
			{
				waitTime--;
				sprFloater.play("Fall");
				gravity();
			}
			else
			{
				sprFloater.play("Float");
				if (player)
				{
					sndOFloaterFloat.volume = Math.max(1 - FP.distance(x, y, player.x, player.y) / 500, 0);
					if (!FP.world.collideLine("Solid", x, y, player.x, player.y) && FP.distance(x, y, player.x, player.y) <= range)
					{
						acceleration.x = FP.sign(player.x - x) * Math.abs(accelMax.x);
						acceleration.y = FP.sign(player.y - 64 - y) * Math.abs(accelMax.y);
						if (Math.abs(player.x - x) < 20)
						{
							falling = true;
						}
					}
					else
					{
						acceleration.y = -accelMax.y;
					}
				}
			}
			if (Math.abs(speed.x) >= mMaxspeed.x)
			{
				sprFloater.scaleX = FP.sign(speed.x) * Math.abs(sprFloater.scaleX);
			}
			friction(true, false);
			maxspeed();
			prevSpeed = new Point(speed.x, speed.y);
			motion();
			if (y - height / 2 >= FP.height)
			{
				FP.world.remove(this);
			}
		}
		
		public function lightDraw():void
		{
			Draw.setTarget(Game.lights);
			sprFloaterLights.frame = sprFloater.frame;
			sprFloaterLights.scaleX = sprFloater.scaleX;
			sprFloaterLights.blend = BlendMode.ADD;
			sprFloaterLights.alpha = 0.5;
			sprFloaterLights.render(new Point(x, y), FP.camera);
			Draw.resetTarget();
		}
		
	}

}