package source.Objects 
{
	import flash.geom.Point;
	import net.flashpunk.graphics.Spritemap;
	import net.flashpunk.FP;
	import net.flashpunk.Sfx;
	import source.View;
	import net.flashpunk.utils.Draw;
	import source.Game;
	import flash.display.BlendMode;
	/**
	 * ...
	 * @author Time
	 */
	public class Flyer extends Enemy
	{
		[Embed(source = "../../assets/sounds/FlyerFly.mp3")] private var sndFlyerFly:Class;
		private var sndOFlyerFly:Sfx = new Sfx(sndFlyerFly);
		[Embed(source = "../../assets/sounds/Flyer.mp3")] private var sndFlyer:Class;
		private var sndOFlyer:Sfx = new Sfx(sndFlyer);
		
		[Embed(source = "../../assets/graphics/flyerLights.png")] private var imgFlyerLights:Class;
		private var sprFlyerLights:Spritemap = new Spritemap(imgFlyerLights, 45, 52, endAnim);
		[Embed(source = "../../assets/graphics/flyer.png")] private var imgFlyer:Class;
		private var sprFlyer:Spritemap = new Spritemap(imgFlyer, 45, 52, endAnim);
		
		private const accelMax:Point = new Point(0.6, 0.2);
		
		private var falling:Boolean = false;
		
		private var range:int = 300;
		
		private var waitTime:int = 0;
		private const waitTimeMax:int = 120;
		
		public function Flyer(_x:int, _y:int) 
		{
			super(_x, _y);
			graphic = sprFlyer;
			type = "Flyer";
			
			sprFlyer.add("Fly", [2, 3, 4, 3], 1);
			sprFlyer.add("Stab", [2, 1, 0], 0.1);
			sprFlyer.add("Stabbing", [0]);
			sprFlyer.add("Hurt", [5, 0, 5, 0, 5], 0.1);
			sprFlyer.play("Fly");
			sprFlyer.originX = 22;
			sprFlyer.originY = 36;
			sprFlyer.x = -sprFlyer.originX;
			sprFlyer.y = -sprFlyer.originY;
			
			sprFlyerLights.originX = 22;
			sprFlyerLights.originY = 36;
			sprFlyerLights.x = -sprFlyerLights.originX;
			sprFlyerLights.y = -sprFlyerLights.originY;
			
			mMaxspeed.x = 3;
			
			setHitbox(24, 24, 12, 12);
			
			sndOFlyerFly.loop();
		}
		
		override public function removed():void
		{
			sndOFlyerFly.stop();
		}
		
		override public function update():void
		{
			var player:Player = FP.world.nearestToEntity("Player", this) as Player;
			acceleration = new Point();
			sndOFlyerFly.volume = 0;
			if (falling)
			{
				sprFlyer.angle = (sprFlyer.angle + 360) % 360;
				sprFlyer.angle += 180 / Math.PI * FP.angle_difference(Math.atan2(-speed.y, speed.x) - Math.PI / 2, sprFlyer.angle * Math.PI / 180) / 6;
				gravity();
				if (collide("Solid", x, y + 1))
				{
					falling = false;
					waitTime = waitTimeMax;
					View.time = Math.abs(speed.y);
					if (player)
					{
						sndOFlyer.play(Math.max(1 - FP.distance(x, y, player.x, player.y) / 500, 0));
					}
					else
					{
						sndOFlyer.play(0.25);
					}
				}
			}
			else if (waitTime > 0)
			{
				waitTime--;
				gravity();
				sprFlyer.angle = (sprFlyer.angle + 360) % 360;
				sprFlyer.angle += 180 / Math.PI * FP.angle_difference(Math.atan2(-speed.y, speed.x) - Math.PI / 2, sprFlyer.angle * Math.PI / 180) / 6;
			}
			else if(!hit)
			{
				sprFlyer.play("Fly");
				sprFlyer.angle += 180 / Math.PI * FP.angle_difference(0, sprFlyer.angle * Math.PI / 180) / 6;
				if (player)
				{
					sndOFlyerFly.volume = 0.35 * Math.max(1 - FP.distance(x, y, player.x, player.y) / 500, 0);
					if (!FP.world.collideLine("Solid", x, y, player.x, player.y) && FP.distance(x, y, player.x, player.y) <= range)
					{
						acceleration.x = FP.sign(player.x - x) * Math.abs(accelMax.x);
					}
				}
				acceleration.y = -Math.abs(accelMax.y);
				speed.y = Math.max(speed.y, -1);
				if (collide("Solid", x, y - 1))
				{
					falling = true;
					sprFlyer.play("Stab");
				}
			}
			friction(true, false);
			motion();
			
			if (y - height / 2 >= FP.height)
			{
				FP.world.remove(this);
			}
		}
		
		override public function endAnim():void
		{
			super.endAnim();
			if (sprFlyer.currentAnim == "Stab")
			{
				sprFlyer.play("Stabbing");
			}
		}
		
		public function lightDraw():void
		{
			Draw.setTarget(Game.lights);
			sprFlyerLights.angle = sprFlyer.angle;
			sprFlyerLights.frame = sprFlyer.frame % sprFlyerLights.frameCount;
			sprFlyerLights.scaleX = sprFlyer.scaleX;
			sprFlyerLights.blend = BlendMode.ADD;
			sprFlyerLights.alpha = 0.5;
			sprFlyerLights.render(new Point(x, y), FP.camera);
			Draw.resetTarget();
		}
		
	}

}