package source.Objects 
{
	import net.flashpunk.Entity;
	import net.flashpunk.FP;
	import net.flashpunk.graphics.Image;
	import source.Global;
	/**
	 * ...
	 * @author Noel Berry
	 */
	public class Moving extends Physics
	{
		[Embed(source = '../../assets/graphics/moving.png')] public var imgMoving:Class;
		public var sprMoving:Image = new Image(imgMoving);
		
		public var direction:Boolean = FP.choose(false);
		public var movement:Number = 2;
		public var carry:Array = new Array("Solid", "Player");
		
		public function Moving(x:int, y:int) 
		{
			//our x/y position
			super(x, y);
			
			//graphic & hitbox
			graphic = sprMoving;
			sprMoving.x = -2;
			sprMoving.y = -2;
			setHitbox(64, 32);
		}
		
		override public function update():void {
			
			//move in the correct direction
			speed.x = direction ? movement : - movement;
			
			//move stuff that's on top of us, for each type of entity we can carry
			for each(var i:String in carry) {
				moveontop(i,speed.x);
			}
			
			//move ourselves
			motion();
			
			//if we've stopped moving, switch directions!
			if ( speed.x == 0 ) { direction = !direction; }
		}
		
		/**
		 * Moves the set entity horizontal at a given speed, checking for collisions and slopes
		 * @param	e		The entity you want to move
		 * @param	spdx	The speed at which the entity should move
		 * @return	True (didn't hit a solid) or false (hit a solid)
		 */
		override public function motionx(e:Entity, spdx:Number):Boolean
		{
			//check each pixel before moving it
			for (var i:int = 0; i <= Math.abs(spdx); i ++) 
			{
				//if we've moved
				var moved:Boolean = false;
				var below:Boolean = true;
				
				if (!e.collide(solid, e.x, e.y + 1)) { below = false; }
				
				//run through how high a slope we can move up
				for (var s:int = -slopeHeight * int(below); s <= slopeHeight; s ++)
				{
					//if we don't hit a solid in the direction we're moving, move....
					if (!e.collide(solid, e.x + FP.sign(spdx), e.y - s)) 
					{
						//increase/decrease positions
						//if the player is in the way, simply don't move (but don't count it as stopping)
						if (!e.collide(Global.player.type, e.x + FP.sign(spdx), e.y - s)) { e.x += FP.sign(spdx) * Math.min(Math.abs(spdx) - i, 1); }
						
						//move up the slope
						e.y -= s;
						
						//we've moved
						moved = true;
						
						//stop checking for slope (so we don't fly up into the air....)
						break;
					}
					
				}
				
				//if we are now in the air, but just above a platform, move us down.
				if (below && !e.collide(solid,e.x, e.y + 1)) { e.y += 1; }
				
				//if we haven't moved, set our speed to 0
				if ( !moved ) { return false; }
				
			}
			
			//hit nothing!
			return true;
		}
		
		/**
		 * Moves the set entity vertical at a given speed, checking for collisions
		 * @param	e		The entity you want to move
		 * @param	spdy	The speed at which the entity should move
		 * @return	True (didn't hit a solid) or false (hit a solid)
		 */
		override public function motiony(e:Entity, spdy:Number):Boolean
		{
			//for each pixel that we will move...
			for ( var i:int = 0; i < Math.abs(spdy); i ++ )
			{
				//if we DON'T collide with solid
				if (!e.collide(solid, e.x, e.y + FP.sign(spdy))) { 
					//if we don't run into a player, them move us
					if (!e.collide(Global.player.type, e.x, e.y + FP.sign(spdy))) { e.y += FP.sign(spdy) * Math.min(Math.abs(spdy) - i, 1); }
					//but note that we wont stop our movement if we hit a player.
				} else { 
					//stop movement if we hit a solid
					return false; 
				}
			}
			
			//hit nothing!
			return true;
		}
		
	}

}