package source.Objects 
{
	import flash.geom.Point;
	import net.flashpunk.Entity;
	import net.flashpunk.graphics.Image;
	import net.flashpunk.graphics.Spritemap;
	import net.flashpunk.Sfx;
	import net.flashpunk.utils.Input;
	import net.flashpunk.utils.Key;
	import source.Global;
	import net.flashpunk.FP;
	import source.Game;
	import source.Menu;
	import source.Transition;
	
	public class Player extends Physics
	{
		[Embed(source = "../../assets/sounds/Die.mp3")] private var sndDie:Class;
		private var sndODie:Sfx = new Sfx(sndDie);
		[Embed(source = "../../assets/sounds/Dive.mp3")] private var sndDive:Class;
		private var sndODive:Sfx = new Sfx(sndDive);
		
		[Embed(source = '../../assets/graphics/player.png')] public var imgPlayer:Class;
		public var sprPlayer:Spritemap = new Spritemap(imgPlayer, 20, 21, animEnd);
		
		//how fast we accelerate
		public var movement:Number = 1;
		public var jump:Number = 7.5;
		
		public var direction:int = 1;
		
		//are we on the ground?
		public var onground:Boolean = false;
		
		public var dead:Boolean = false;
		public var start:Point;
		
		private var myLight:Light;
		
		private var flying:Boolean = false;
		private var flyingVelocity:Point = new Point(5, -3);
		private var gettingUp:Boolean = false;
		
		private var trapped:Boolean = false;
		private var trappedPos:Point = new Point();
		private var angSpin:Number = 0;
		
		private const sleepingTimeMax:int = 120;
		private var sleepingTime:int = -1;
		
		private var intro:Boolean = true;
		private var outro:Boolean = false;
		private var outro2:Boolean = false;
		public var leprechaunStopped:Boolean = false;
		
		public function Player(x:int, y:int) 
		{
			//set position
			super(x + 16, y + 18);
			start = new Point(x, y);
			
			//set different speeds and such
			mGravity = 0.4;
			mMaxspeed = new Point(2, 8);
			mFriction = new Point(0.5, 0.5);
			
			//set up animations
			sprPlayer.add("stand", [0,0,1], 0.03);
			sprPlayer.add("walk", [2, 3, 4, 5, 6, 7, 8, 9], 0.3);
			
			sprPlayer.add("jumpDown", [15]);
			sprPlayer.add("jumpUp", [16]);
			
			sprPlayer.add("fly", [10, 11, 12], 0.5);
			sprPlayer.add("flying", [12]);
			sprPlayer.add("gettingUp", [18, 19, 20, 21], 0.3);
			
			sprPlayer.add("sleeping", [17]);
			sprPlayer.add("wakingUp", [18, 19, 20, 21, 22, 23, 24, 24, 24, 25], 0.1);
			
			sprPlayer.add("hit", [26]);
			
			sprPlayer.centerOO();
			
			//set hitbox & graphic
			setHitbox(8, 17, 4, 7);
			graphic = sprPlayer;
			type = "Player";
			
			myLight = new Light(x, y);
			FP.world.add(myLight);
			
			if (Global.level-1 == 0)
			{
				intro = true;
				sprPlayer.play("sleeping");
				sleepingTime = sleepingTimeMax;
			}
			else
			{
				sprPlayer.play("stand");
			}
			
			if (Game.levels[Global.level - 1] == Game.outro)
			{
				outro = true;
			}
			if (Game.levels[Global.level - 1] == Game.outro2)
			{
				outro2 = true;
				x = FP.width + 64;
			}
			if (outro || outro2)
			{
				speed.x = -mMaxspeed.x;
				direction = -1;
			}
		}
		
		override public function update():void 
		{
			if (myLight)
			{
				myLight.x = x;
				myLight.y = y;
			}
			
			if (sleepingTime > 0)
			{
				sleepingTime--;
			}
			else if(sleepingTime == 0)
			{
				sprPlayer.play("wakingUp");
				sleepingTime = -1;
			}
			
			//did we... die?
			//if (dead) { sprPlayer.alpha -= 0.01; return; } else if ( sprPlayer.alpha < 1 ) { sprPlayer.alpha += 0.01 }
			if (trapped)
			{
				angSpin = Math.min(angSpin + 0.05, 5);
				sprPlayer.play("jumpDown");
				sprPlayer.angle += angSpin;
				x += (trappedPos.x - x) / 10;
				y += (trappedPos.y - 16 - y) / 10;
				return;
			}
			//are we on the ground?
			onground = false;
			if (collide(solid, x, y + 1)) 
			{ 
				onground = true;
				if (sprPlayer.currentAnim == "flying")
				{
					sprPlayer.play("gettingUp");
					gettingUp = true;
					speed.x /= 2;
					flying = false;
				}
			}
			
			//set acceleration to nothing
			acceleration.x = 0;
			acceleration.y = 0;
			
			if (outro || outro2)
			{
				speed.x = -mMaxspeed.x;
				if (outro2)
				{
					sprPlayer.play("walk"); 
					if(x <= FP.width/4)
					{
						speed.x = 0;
						Game.cycleSolids = true;
					}
				}
				if (x <= 32)
				{
					Global.finished = true;
				}
				Global.view.x = x - 160;
				Global.view.y = y - 120;
			}
			else if (x < 0)
			{
				Global.finished = true;
				if (Global.level != Game.levels.length - 2)
				{
					Game.gotoLepRoom = true;
					x = 0;
				}
			}
			
			if (!dead && !leprechaunStopped && !flying && !gettingUp && !outro && !outro2 && sprPlayer.currentAnim != "wakingUp" && sprPlayer.currentAnim != "sleeping")
			{
				//increase acceeration, if we're not going too fast
				if (Input.check(Global.keyLeft) && speed.x > -mMaxspeed.x) { acceleration.x = - movement; direction = -1; }
				if (Input.check(Global.keyRight) && speed.x < mMaxspeed.x) { acceleration.x = movement; direction = 1; }
				
				//friction (apply if we're not moving, or if our speed.x is larger than maxspeed)
				if ( (! Input.check(Global.keyLeft) && ! Input.check(Global.keyRight)) || Math.abs(speed.x) > mMaxspeed.x ) { friction(true, false); }
				
				//jump
				if ( Input.pressed(Global.keyA) || Input.pressed(Global.keyUp)) 
				{					
					//normal jump
					if (onground) { 
						speed.y = -jump;
					}
				}
				
				if (Input.check(Global.keyDive))
				{
					speed.x = flyingVelocity.x * direction;
					speed.y = flyingVelocity.y;
					sprPlayer.play("fly");
					flying = true;
					sndODive.play(0.5);
				}
			}
			
			//set the gravity
			gravity();
			
			//make sure we're not going too fast vertically
			//the reason we don't stop the player from moving too fast left/right is because
			//that would (partially) destroy the walljumping. Instead, we just make sure the player,
			//using the arrow keys, can't go faster than the max speed, and if we are going faster
			//than the max speed, descrease it with friction slowly.
			maxspeed(false, true)
			
			
			//set the sprites according to if we're on the ground, and if we are moving or not
			sprPlayer.scaleX = Math.abs(sprPlayer.scaleX) * FP.sign(direction);
			if (!dead && !flying && !gettingUp && !outro2 && sprPlayer.currentAnim != "sleeping" && sprPlayer.currentAnim != "wakingUp")
			{
				if (onground)
				{
					if (speed.x != 0) 
					{ 
						sprPlayer.play("walk"); 
					}
					
					else 
					{
						sprPlayer.play("stand");
					}
				} 
				else 
				{
					if (speed.y < 0)
					{
						sprPlayer.play("jumpUp");
					}
					else
					{
						sprPlayer.play("jumpDown");
					}
				}
			}
			
			
			//set the motion. We set this later so it stops all movement if we should be stopped
			motion();
			x = Math.round(x);
			y = Math.floor(y);
			
			if (!outro && !outro2)
			{
				//did we just get.. KILLED? D:
				const timeReset:int = 12;
				var c:Entity = collide("Faller", x, y);
				if (c && !dead)
				{
					trapped = true;
					trappedPos = new Point(c.x, c.y);
					killme(x);
				}
				c = collide("Floater", x, y);
				if (c && !dead)
				{
					if(!(c as Enemy).hit)
					{
						killme(c.x);
						(FP.world as Game).reset = timeReset;
					}
				}
				c = collide("Flyer", x, y);
				if (c && !dead)
				{
					if (flying)// || gettingUp)
					{
						(c as Enemy).die();
					}
					else if(!(c as Enemy).hit)
					{
						killme(c.x);
						(FP.world as Game).reset = timeReset;
					}
				}
				c = collide("Worm", x, y);
				if (c && !dead)
				{
					if (flying)// || gettingUp)
					{
						(c as Enemy).die();
					}
					else if(!(c as Enemy).hit)
					{
						killme(c.x);
						(FP.world as Game).reset = timeReset;
					}
				}
				c = collide("Sitter", x, y);
				if (c && !dead)
				{
					if (flying)// || gettingUp)
					{
						(c as Enemy).die();
					}
					else if(!(c as Enemy).hit)
					{
						killme(c.x);
						(FP.world as Game).reset = timeReset;
					}
				}
				c = collide("Spikes", x, y);
				if (c && speed.y > 0 && !dead)
				{
					//killme!
					killme(c.x);
					(FP.world as Game).reset = timeReset;
				}
				if (y > FP.height)
				{
					killme(x);
				}
			}
			
		}
		
		public function killme(c_x:int):void
		{
			if (!dead)
			{
				dead = true;
				Global.restart = true;
				sndODie.play();
				if (c_x != x)
				{
					speed.x = FP.sign(x - c_x) * 4;
					speed.y = -2;
					sprPlayer.play("hit");
				}
			}
		}
		
		public function animEnd():void 
		{ 
			if (sprPlayer.currentAnim == "wakingUp")
			{
				sprPlayer.play("stand");
			}
			if (sprPlayer.currentAnim == "fly")
			{
				sprPlayer.play("flying");
			}
			if (sprPlayer.currentAnim == "gettingUp")
			{
				gettingUp = false;
			}
		}
		
	}

}